;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.include "tn15def.inc"
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.CSEG

; :
; 1 - RESET
; 2 - PB4 - adc
; 3 - PB3 - relay
; 4 - GND

; 5 - PB0 - tx
; 6 - NC
; 7 - PB2 - rx
; 8 - Vcc

;     - R18.

;    .
.equ		INT0_BY_FALLING_EDGE	=(1 << ISC01)

;   ,  UART'.
.equ		UART_TCCR_CK_8		=2
.equ		UART_TIMER_LIM		=173
.equ		UART_TIMER_LIM_x15	=131
;   ,  ADC.
.equ		ADC_TCCR_CK_1024	=15
.equ		ADC_TIMER_LIM		=131
;   ADC.
.equ		ADC_VREF_USE_VCC	=0x00	; 0000 0000
.equ		ADC_VREF_INTERNAL	=0x80	; 1000 0000
.equ		ADC_DIFF_IN_x1		=0x06	; 0000 0110
.equ		ADC_DIFF_IN_x20		=0x07	; 0000 0111
.equ		ADC_SINGLE_PB3		=0x02	; 0000 0010
.equ		ADC_SINGLE_PB4		=0x03	; 0000 0011

; :
.equ		LF					=0x0A
.equ		CR					=0x0D

;    LPM     FLASH .
.def		chr					=R0

;     ,    .
.equ		cmd_lim				=10		;   
.def		cmd_str				=R1		;  -,  
										; ,    = 10,   - R10
.def		cmd_len				=R11	;   

;      .
.def		sreg_copy			=R12

;     .
;  ,     .
.equ		cmd_ok				=0		;  
.equ		cmd_error			=1		;      
.equ		write_t_on			=2		;      EEPROM
.equ		write_t_off			=3		;      EEPROM
.equ		need_convert		=4		;  ,  
.equ		uart_transmitting	=5		; UART   
.equ		uart_receiving		=6		; UART  
.equ		uart_have_data		=7		; UART      

;     .
.equ		cmd_ok_mask				=1 << cmd_ok
.equ		cmd_error_mask			=1 << cmd_error
.equ		write_t_on_mask			=1 << write_t_on
.equ		write_t_off_mask		=1 << write_t_off
.equ		need_convert_mask		=1 << need_convert
.equ		uart_transmitting_mask	=1 << uart_transmitting
.equ		uart_receiving_mask		=1 << uart_receiving
.equ		uart_have_data_mask		=1 << uart_have_data

;     .
.def		flags				=R16	;   

;   .
.def		t_on				=R13	;  
.def		t_off				=R14	;  
.def		t_cur				=R17	;  

;       UART'.
.def		uart_buffer			=R15
.def		uart_bitcnt			=R19

.equ		uart_tx_port		=PORTB
.equ		uart_tx_pin			=PORTB0
.equ		uart_rx_port		=PINB
.equ		uart_rx_pin			=PORTB2

;       .
.def		ADCLow				=R20
.def		ADCHigh				=R21

.equ		adc_port			=PORTB
.equ		adc_pin				=PORTB4
.equ		range_port			=PORTB
.equ		range_pin			=PORTB1

;     .
.def		t1					=R22
.def		t2					=R23
;     .
.def		ti1					=R24
.def		ti2					=R25

;      .
.def		fres				=R26

; -:
.def		cmd_ptr				=R27	;     .
.def		src_ptr				=R28	;   ZL,     FLASH.
.def		dst_ptr				=R29	;         .
;.def		ZL					=R30	; , "" 
;.def		ZH					=R31	;     0!!!

.equ		relay_port			=PORTB
.equ		relay_pin			=PORTB3
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
INT_TABLE:
	rjmp	RESET		; Reset handler
	rjmp	EXT_INT0	; IRQ0 handler
	reti				; Pin change handler
	reti				; Timer1 compare match handler
	rjmp	TIMER1_OVF	; Timer1 overflow handler
	rjmp	TIMER0_OVF	; Timer0 overflow handler
	rjmp	EEPROM_READY; EEPROM Ready handler
	reti				; Analog Comparator handler
	rjmp	ADC_READY	; ADC Conversion handler
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; !   ATtiny15L   SRAM,     (Z),
;        :
;  -       -  ;
;  -           ;
;  -  1: ZH  == 0;
;  -  2:   ZL
;				        256  FLASH .
; :
msg_ok:		.db	"Ok.", 0
msg_err:	.db	"Error", 0
msg_t:		.db	"T=", 0
; :
cmd_t:		.db	"?", 0
cmd_t_on:	.db	"ON=", 0
cmd_t_off:	.db	"OF=", 0
cmd_state:	.db	"STATE?", 0
cmd_addrs:	.db	LOW(2*cmd_t_on), LOW(2*cmd_t_off), LOW(2*cmd_state), LOW(2*cmd_t), 0
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;         .
;  (RT)    R1 = 15   GND.
;     Vcc.        RT  R1.
;   ,       
;     5 .
dADCtbl:
	.db		5, 5, 8, 10, 12, 15, 18, 22, 26, 31, 35, 40, 44, 48, 49, 51
	.db		52, 51, 50, 47, 45, 40, 38, 33, 30, 26, 24, 20
dADCtblEnd:

.equ		MinT			=-20	;   ,       MinADC.
.equ		MinADC			=12		;      .
.equ		MaxT			=125	;   ,    .
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
EXT_INT0:
	;   
	in		sreg_copy, SREG
	;  
	sbr			flags, uart_receiving_mask
	;     
	ldi		ti1, UART_TIMER_LIM_x15
	out		TCNT0, ti1
	;    
	clr		ti1
	out		GIMSK, ti1
	;     
	ldi		ti1, (1 << TOV0)
	out		TIFR, ti1
	;     0
	in		ti1, TIMSK
	sbr		ti1, (1 << TOIE0)
	out		TIMSK, ti1
	;     
	clr		uart_buffer
	;     (   !!!)
	ldi		uart_bitcnt, 1
	;   
	out		SREG, sreg_copy
	reti
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
TIMER0_OVF:
	;   
	in		sreg_copy, SREG
	;    /?
	cpi		uart_bitcnt, 9
	breq	all_done
	; ,  ,     
	ldi		ti1, UART_TIMER_LIM
	out		TCNT0, ti1

	;  
	sbrc	flags, uart_transmitting
	rjmp	timer0_transmit

	sbrc	flags, uart_receiving
	rjmp	timer0_receive

	;  ,    ,     
	rjmp	timer0_exit

timer0_receive:
	lsr		uart_buffer			;   ,   - ,     
	in		ti1, uart_rx_port	;    
	bst		ti1, uart_rx_pin	;   ,   SREG[T]
	bld		uart_buffer, 7		;     
	rjmp	tx_rx_exit

timer0_transmit:
	;    
	cpi		uart_bitcnt, 8
	brne	transmit_data
	;   ,  ,  
	sbi		uart_tx_port, uart_tx_pin
	rjmp	tx_rx_exit

transmit_data:					;     LSB
	in		ti1, uart_tx_port	;    
	bst		uart_buffer, 0		;   ,    (  SREG[T])
	bld		ti1, uart_tx_pin	;    
	out		uart_tx_port, ti1	;      
	lsr		uart_buffer			;    ,   
	rjmp	tx_rx_exit

;    ,  /
all_done:
	;     0
	in		ti1, TIMSK
	cbr		ti1, (1 << TOIE0)
	out		TIMSK, ti1
	;   
	ldi		uart_bitcnt, 0xFF
	;     (  " ")
	bst		flags, uart_receiving		;     "",     SREG[T]
	cbr		flags, uart_transmitting_mask | uart_receiving_mask	;      (,       )
	brtc	tx_rx_exit					;   SREG[T]  ,    
	;  ,  ,  
	; ,       
	ldi		ti1, cmd_lim
	cp		cmd_len, ti1
	brne	skip_len_error
	sbr		flags, cmd_error_mask
	rjmp	tx_rx_exit
skip_len_error:
	; ,     ?
	ldi		ti1, CR
	cp		uart_buffer, ti1
	brne	skip_eol
	;  
	sbr		flags, cmd_ok_mask
	clr		uart_buffer

	rjmp	skip_enable_pin		;    ...
								;     ,     
skip_eol:
	;      
	ldi		ti1, (1 << INTF0)
	out		GIFR, ti1
	;     (-   ,    )
	ldi		ti1, (1 << INT0)
	out		GIMSK, ti1
skip_enable_pin:
	;   
	mov		ti1, ZL		;  ,     
	inc		cmd_len		;    
						; (  R0     LPM      ,   )
	mov		ZL, cmd_len
	st		Z, uart_buffer
	mov		ZL, ti1		;  

tx_rx_exit:
	inc		uart_bitcnt

timer0_exit:
	;   
	out		SREG, sreg_copy
	reti
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
TIMER1_OVF:
	;   
	in		sreg_copy, SREG

	; ,  ,     
	ldi		ti1, ADC_TIMER_LIM
	out		TCNT1, ti1

	;  ADC  
	sbi		ADCSR, ADSC

	;   
	out		SREG, sreg_copy
	reti
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
ADC_READY:
	;   
	in		sreg_copy, SREG

	; ,    .     ,
	;      ADC .
	in		ADCLow, ADCL
	in		ADCHigh, ADCH
	; ,    .
	sbr		flags, need_convert_mask

	;   
	out		SREG, sreg_copy
	reti
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
EEPROM_READY:
	;   
	in		sreg_copy, SREG
	; ,   ?
	sbrc	flags, write_t_on
	rjmp	eeprom_write_t_on
	sbrc	flags, write_t_off
	rjmp	eeprom_write_t_off
	;    ,  .
	cbi		EECR, EERIE
	rjmp	eeprom_ready_exit

eeprom_write_t_on:
	cbr		flags, write_t_on_mask
	mov		ti1, t_on
	ldi		ti2, T_ON_STORE
	rjmp	eeprom_write
eeprom_write_t_off:
	cbr		flags, write_t_off_mask
	mov		ti1, t_off
	ldi		ti2, T_OFF_STORE
eeprom_write:
	out		EEAR, ti2
	out		EEDR, ti1
	sbi		EECR, EEMWE
	sbi		EECR, EEWE

eeprom_ready_exit:
	;   
	out		SREG, sreg_copy
	reti
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
uart_transmit:
	;  
	cbr		flags, uart_receiving_mask
	sbr		flags, uart_transmitting_mask
	;    
	clr		uart_bitcnt
	;   
	cbi		uart_tx_port, uart_tx_pin
	;     
	ldi		t1, UART_TIMER_LIM
	out		TCNT0, t1
	;     
	ldi		t1, (1 << TOV0)
	out		TIFR, t1
	;     0
	in		t1, TIMSK
	sbr		t1, (1 << TOIE0)
	out		TIMSK, t1
	; ,     
wait_uart_free:
	sbrc	flags, uart_transmitting	;     
	rjmp	wait_uart_free				;   
	; 
	ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;      .
disable_pin:
	;    
	clr		t1
	out		GIMSK, t1
	; 
	ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;      .
enable_pin:
	;      
	ldi		t1, (1 << INTF0)
	out		GIFR, t1
	;    
	ldi		t1, (1 << INT0)
	out		GIMSK, t1
	; 
	ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;   .      = 1.
;       .
send_str:
	rcall	disable_pin
	ldi		ZL, 1
ss_loop:
	ld		uart_buffer, Z
	tst		uart_buffer
	breq	out_ss
	rcall	uart_transmit
	inc		ZL
	rjmp	ss_loop
out_ss:
	ldi		t1, CR
	mov		uart_buffer, t1
	rcall	uart_transmit
	ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;   .    src_ptr    FLASH .
;  dst_ptr -     .
;   STR_EQUAL(1)  0 -    .
.equ		STR_EQUAL			=1
; ,     ,     
;   /,      .
strcmp:
	;    
	mov		ZL, src_ptr
	lpm				; chr <- [Z]
	;    
	tst		chr
	breq	str_end
	;   
	mov		ZL, dst_ptr
	ld		t1, Z
	;  
	cp		t1, chr
	brne	str_not_equal
	;  
	inc		src_ptr
	inc		dst_ptr
	; 
	rjmp	strcmp

str_not_equal:
	clr		fres			;   
	ret

str_end:
	ldi		fres, STR_EQUAL	;  
	ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;     FLASH-   src_ptr,
;      dst_ptr.
strcpy:
	mov		ZL, src_ptr
	lpm			; chr <- [Z]
	mov		ZL, dst_ptr
	st		Z, chr
	tst		chr
	breq	cpy_exit
	inc		src_ptr
	inc		dst_ptr
	rjmp	strcpy
cpy_exit:
	ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;     .
;   -     dst_ptr.   fres.
atoi:
	; 
	mov		ZL, dst_ptr
	clr		fres
	clt				; T = 0
	;     
	ld		t1, Z
	cpi		t1, '-'
	brne	skip_sign
	;  ,   
	inc		ZL
	set				; T = 1

skip_sign:
	;   
	ld		t1, Z
	inc		ZL
	subi	t1, '0'		;  -> 
	brlt	atoi_end	;  < 0?     
	cpi		t1, 9		;  > 9?     
	brpl	atoi_end
	;   10 
	lsl		fres
	mov		t2, fres
	lsl		fres
	lsl		fres
	add		fres, t2
	;    
	add		fres, t1
	; 
	rjmp	skip_sign

atoi_end:
	brtc	atoi_exit
	;    
	neg		fres
atoi_exit:
	ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;     .
;     fres,     dst_str.
itoa:
	mov		ZL, dst_ptr
	ldi		t1, '0'

	tst		fres
	breq	skip10	;  ,    .
	brpl	skip_sign_out	;  < 0,   .
	ldi		t2, '-'
	st		Z, t2
	inc		ZL
	neg		fres
skip_sign_out:
	clr		t2
	;  ,   > 100.
	cpi		fres, 100
	brlo	skip100
sub100_loop:
	cpi		fres, 100
	brlo	sub100_stop
	subi	fres, 100
	inc		t2
	rjmp	sub100_loop
sub100_stop:
	add		t2, t1
	st		Z, t2
	inc		ZL
	clr		t2
	rjmp	sub10_loop
	;  ,   > 10.
skip100:
	cpi		fres, 10
	brlo	skip10
sub10_loop:
	cpi		fres, 10
	brlo	sub10_stop
	subi	fres, 10
	inc		t2
	rjmp	sub10_loop
sub10_stop:
	add		t2, t1
	st		Z, t2
	inc		ZL

skip10:
	add		fres, t1
	st		Z, fres
	inc		ZL
	;  0 (ASCIIZ ).
	clr		fres
	st		Z, fres
	; .
	ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
toupper:
	mov		ZL, dst_ptr
	ldi		t2, 10

toupper_loop:
	ld		t1, Z
	tst		t1
	breq	toupper_exit
	cpi		t1, 'a'
	brmi	skip_upper
	cpi		t1, 'z' + 1
	brpl	skip_upper

	subi	t1, 'a' - 'A'
	st		Z, t1
skip_upper:
	inc		ZL
	dec		t2
	brne	toupper_loop

toupper_exit:
	ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;         .
;     ,   ,   .
;     0  255.      0  4.
;      .    
; " ".
;  :
; chr - da -         5 .
; ADCLow - adc -    .
;   t_cur.
Bresenham:
	mov		fres, ADCLow
	ldi		t1, 5
	sub		chr, t1		; da -= dt
	clr		ADCLow		; d = 0
	clr		ADCHigh
	clr		t1
	clr		t2
bres_loop:				; do {
	cp		ADCLow, t1	; 	if (d < 0)
	cpc		ADCHigh, t2
	brmi	less_than_zero	;     then
						; 	else d -= 2*da
	sub		ADCLow, chr	; 		d -= da
	sbc		ADCHigh, t2
	sub		ADCLow, chr	; 		d -= da
	sbc		ADCHigh, t2
	rjmp	check_d
less_than_zero:			; 	then
	subi	ADCLow, -10	; 		d += 2*dt	; LOW(-10)
	sbci	ADCHigh, 0xFF					; HIGH(-10)
check_d:
	cp		ADCLow, t1	; 	if (d >= 0)
	cpc		ADCHigh, t2
	brmi	skip_inc_t
	inc		t_cur		; 		then t++
skip_inc_t:
	dec		fres		;	adc--
	tst		fres		;
	brne	bres_loop	; } while (adc > 0);
bres_exit:
	ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;   .
;   - ADCHigh:ADCLow.
;   t_cur.
ConvertT:
	; ,   ,     ;)
	ldi		t_cur, MinT		;    
	clr		fres

	ldi		t1, MinADC
	ldi		t2, 0
	cp		ADCLow, t1
	cpc		ADCHigh, t2
	brlo	conv_exit		; ,   
	;  >= FirstT,  .
	ldi		ZL, LOW(2*dADCtbl)
conv_loop:
	lpm						; chr = [Z]
	add		t1, chr			; tblADC += dADC
	adc		t2, fres		; fres == 0!!!
	cp		ADCLow, t1		; realADC < tblADC?
	cpc		ADCHigh, t2
	brlo	range_founded	; ,    
	subi	t_cur, -5
	inc		ZL				; ,     
	cpi		ZL, LOW(2*dADCtblEnd)
	brne	conv_loop
	;   ,   > 125 .
	ldi		t_cur, 125
	rjmp	conv_exit

range_founded:
	sub		t1, chr			;   ,     
	sbc		t2, fres
	sub		ADCLow, t1		;      ,
	sbc		ADCHigh, t2		;           
	breq	conv_exit		;     ,   
	rcall	Bresenham		;  ADCLow     , chr -  
	;     t_cur    .
conv_exit:
	ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;    ,        !!!
CheckT:
	sbic	relay_port, relay_pin	;  ?
	rjmp	check_t_off			;     .
	;  . ,    .
	cp		t_cur, t_on
	brsh	check_t_exit		;  ,    .
	sbi		relay_port, relay_pin	;   .
	rjmp	check_t_exit

check_t_off:
	;  . ,     .
	cp		t_cur, t_off
	brlo	check_t_exit		; ,  .    .
	cbi		relay_port, relay_pin	;  .

check_t_exit:
	ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Init:
	;    .
	ldi		t1, 0x84
	out		OSCCAL, t1

	;     /.
	ldi		t1, T_ON_STORE
	out		EEAR, t1
	sbi		EECR, EERE
	in		t_on, EEDR
	ldi		t1, T_OFF_STORE
	out		EEAR, t1
	sbi		EECR, EERE
	in		t_off, EEDR
	;   .       .
	clr		t_cur
	ldi		ADCLow, 3
	ldi		ADCHigh, 4

	;   3   .
	sbi		DDRB, relay_pin
	;   uart'  .
	sbi		DDRB, uart_tx_pin
	;   -  .
	sbi		uart_tx_port, uart_tx_pin

	;    .
	clr		cmd_len
	clr		flags
	;  -.
	clr		ZH		;     0!!!

	;    0.
	ldi		t1, UART_TCCR_CK_8
	out		TCCR0, t1
	;    1.
	ldi		t1, ADC_TCCR_CK_1024
	out		TCCR1, t1
	;     1.
	in		t1, TIMSK
	sbr		t1, (1 << TOIE1)
	out		TIMSK, t1
	; ,         .   pull-up   .
	in		t1, MCUCR
	ori		t1, INT0_BY_FALLING_EDGE + (1 << PUD)
	out		MCUCR, t1
	;  ADC.
	ldi		t1, ADC_VREF_USE_VCC | ADC_SINGLE_PB4
	out		ADMUX, t1		;     PB4    2.56.
	ldi		t1, (1 << ADEN) | (1 << ADIE) | (1 << ADPS2)
	out		ADCSR, t1

	;   .
	sei

	; .
	ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;   .
;    dst_ptr -    .
;  ,   fres     ,
;     ( ,    enum -  ).
find_cmd:
	ldi		cmd_ptr, LOW(2*cmd_addrs)	;      
cmd_search:
	mov		ZL, cmd_ptr			;   "" 
	lpm							;    
	tst		chr					; !!!! LPM    R0,     chr !!!!
	breq	cmd_not_found		;   == 0,   

	mov		src_ptr, chr		;     
	ldi		dst_ptr, 1			;    
	rcall	strcmp				;      
	cpi		fres, STR_EQUAL		;   
	breq	cmd_founded			;     
	inc		cmd_ptr				;        
	rjmp	cmd_search			; 
	;    ,      ,  ,   .
cmd_not_found:
	clr		fres
	ret		;  1.
cmd_founded:					;  
	mov		ZL, cmd_ptr			;      ,        
	lpm							;    R0 (chr),    LPM
	mov		fres, chr
	ret		;  2.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
make_param_str:
	ldi		dst_ptr, 1
	rcall	strcpy
	rcall	itoa
	ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
RESET:
	rcall	Init

	ldi		src_ptr, LOW(2*msg_ok)
	rjmp	out_state_info

clear_and_wait_for_cmd:
	ldi		dst_ptr, 1		;      
	rcall	strcpy			;    
	rcall	send_str		;       
	cbr		flags, cmd_ok_mask | cmd_error_mask
	clr		cmd_len
	rcall	enable_pin

wait_for_cmd:
	;    .
	sbrs	flags, need_convert	;   ,  ,   .
	rjmp	skip_process_t		;  .
	;   ,   ,    .
	rcall	ConvertT
	rcall	CheckT
	cbr		flags, need_convert_mask	;      
	;   .
skip_process_t:
	sbrc	flags, cmd_error
	rjmp	process_error
	sbrs	flags, cmd_ok
	rjmp	wait_for_cmd

process_cmd:
	ldi		dst_ptr, 1
	rcall	toupper
	rcall	find_cmd

	cpi		fres, LOW(2*cmd_t)	; " "
	breq	out_temperature
	cpi		fres, LOW(2*cmd_t_on); "  "
	breq	set_t_on
	cpi		fres, LOW(2*cmd_t_off); "  "
	breq	set_t_off
	cpi		fres, LOW(2*cmd_state); "   "
	breq	out_state_info

	rjmp	process_error		;   ,    

;     -  
out_temperature:
	mov		fres, t_cur
	ldi		dst_ptr, 1
	rcall	itoa
	rcall	send_str
	rjmp	out_confirm

;     
set_t_on:
	;     strcmp:         :)
	mov		src_ptr, fres
	ldi		dst_ptr, 1
	rcall	strcmp
	rcall	atoi
	mov		t_on, fres
	sbr		flags, write_t_on_mask
	sbi		EECR, EERIE ;     
	rjmp	out_confirm

;     
set_t_off:
	mov		src_ptr, fres
	ldi		dst_ptr, 1
	rcall	strcmp
	rcall	atoi
	mov		t_off, fres
	sbr		flags, write_t_off_mask
	sbi		EECR, EERIE ;     
	rjmp	out_confirm

;    
out_state_info:
	ldi		src_ptr, LOW(2*cmd_t_on)
	mov		fres, t_on
	rcall	make_param_str
	rcall	send_str

	ldi		src_ptr, LOW(2*cmd_t_off)
	mov		fres, t_off
	rcall	make_param_str
	rcall	send_str

	ldi		src_ptr, LOW(2*msg_t)
	mov		fres, t_cur
	rcall	make_param_str
	rcall	send_str

out_confirm:
	ldi		src_ptr, LOW(2*msg_ok)
	rjmp	clear_and_wait_for_cmd

process_error:
	ldi		src_ptr, LOW(2*msg_err)
	rjmp	clear_and_wait_for_cmd
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;   .
.ESEG
T_ON_STORE:	.db		20
T_OFF_STORE:.db		30
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
